"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("./base");
var graphql_1 = require("graphql");
var memory_cache_1 = require("memory-cache");
var graphql_tools_1 = require("graphql-tools");
function transform(finalSchema) {
    return base_1.base(function (req, res, next) {
        req.qewl.on('schemaGenerated', function (generatedSchema) {
            if (!memory_cache_1.get('qewl.finalSchema')) {
                var doc = graphql_1.parse(finalSchema);
                // Merge, taking the definitions from the final schema.
                // Unfortunately, this breaks resolvers (https://github.com/apollographql/graphql-tools/issues/504)
                // And it doesn't apply onTypeConflict on the entire Query/Mutation types, just to individual fields
                var schemaDef = graphql_tools_1.mergeSchemas({
                    schemas: [req.qewl.schemas.mergedSchema, finalSchema],
                    onTypeConflict: function (l, r) { return r; }
                });
                // Restore the resolvers for the query type, or remove it or any of its fields if not part of the final schema
                var queryType_1 = schemaDef.getQueryType();
                if (queryType_1) {
                    if (!doc.definitions.map(function (d) { return d.name.value; }).includes(queryType_1.name)) {
                        delete schemaDef._queryType;
                    }
                    else {
                        var queryFields = queryType_1.getFields();
                        var queryFieldKeys = Object.keys(queryFields);
                        var newQueryTypeFields = doc.definitions.find(function (d) { return d.name.value === queryType_1.name; }).fields.map(function (f) { return f.name.value; });
                        for (var _i = 0, queryFieldKeys_1 = queryFieldKeys; _i < queryFieldKeys_1.length; _i++) {
                            var key = queryFieldKeys_1[_i];
                            if (newQueryTypeFields.includes(key)) {
                                queryFields[key].resolve = req.qewl.schemas.mergedSchema.getQueryType().getFields()[key].resolve;
                            }
                            else {
                                delete schemaDef.getQueryType().getFields()[key];
                            }
                        }
                    }
                }
                // Restore the resolvers for the mutation type, or remove it or any of its fields if not part of the final schema
                var mutationType_1 = schemaDef.getMutationType();
                if (mutationType_1) {
                    if (!doc.definitions.map(function (d) { return d.name.value; }).includes(mutationType_1.name)) {
                        delete schemaDef._mutationType;
                    }
                    else {
                        var mutationFields = mutationType_1.getFields();
                        var mutationFieldKeys = Object.keys(mutationFields);
                        var newMutationTypeFields = doc.definitions.find(function (d) { return d.name.value === mutationType_1.name; }).fields.map(function (f) { return f.name.value; });
                        for (var _a = 0, mutationFieldKeys_1 = mutationFieldKeys; _a < mutationFieldKeys_1.length; _a++) {
                            var key = mutationFieldKeys_1[_a];
                            if (newMutationTypeFields.includes(key)) {
                                mutationFields[key].resolve = req.qewl.schemas.mergedSchema.getMutationType().getFields()[key].resolve;
                            }
                            else {
                                delete schemaDef.getMutationType().getFields()[key];
                            }
                        }
                    }
                }
                // Restore the resolvers for the subscription type, or remove it or any of its fields if not part of the final schema
                var subscriptionType_1 = schemaDef.getSubscriptionType();
                if (subscriptionType_1) {
                    if (!doc.definitions.map(function (d) { return d.name.value; }).includes(subscriptionType_1.name)) {
                        delete schemaDef._subscriptionType;
                    }
                    else {
                        var subscriptionFields = subscriptionType_1.getFields();
                        var subscriptionFieldKeys = Object.keys(subscriptionFields);
                        var newSubscriptionTypeFields = doc.definitions.find(function (d) { return d.name.value === subscriptionType_1.name; }).fields.map(function (f) { return f.name.value; });
                        for (var _b = 0, subscriptionFieldKeys_1 = subscriptionFieldKeys; _b < subscriptionFieldKeys_1.length; _b++) {
                            var key = subscriptionFieldKeys_1[_b];
                            if (newSubscriptionTypeFields.includes(key)) {
                                subscriptionFields[key].resolve = req.qewl.schemas.mergedSchema.getSubscriptionType().getFields()[key].resolve;
                            }
                            else {
                                delete schemaDef.getSubscriptionType().getFields()[key];
                            }
                        }
                    }
                }
                req.qewl.schemas.finalSchema = schemaDef;
                memory_cache_1.put("qewl.finalSchema", schemaDef);
            }
            else {
                req.qewl.schemas.finalSchema = memory_cache_1.get("qewl.finalSchema");
            }
        });
        next();
    });
}
exports.transform = transform;
//# sourceMappingURL=transform.js.map